'========================================================================================
'                          .                 .
'                            ______ ______ .
'                         .:_\_ .  \\_ .__\_::.
'                      . .::./ ./  // ./__ .:::. .
'                         :_<_____/<______>_:.
'                         .                 .
'                   Damaged Cybernetics Laboratories
'
'    It is a crime to redistribute these routines in a commercial
'    venture of any kind without permission or a licensing agreement.
'    Contact us via email for more information on licensing.
'
'    This is freely distributable for non-commercial use, however we
'    require that you acknowledge the following:
'
'    Copyright (c) 1999 Damaged Cybernetics. All rights reserved.
'
'    If you have any questions or comments, please contact any of the
'    following people via the Internet:
'
'      Donald Moore    (MindRape)    moore@futureone.com
'========================================================================================
'Title:
'   Bit-Shifting
'Author:
'   Donald Moore (MindRape)
'Purpose:
'   Left and Right bit shifting operators
'========================================================================================



'Enumeration of bit-shifting
Public Enum dcShiftDirection
    Left = -1
    Right = 0
End Enum


'========================================================================================
'Public Function Shift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long, ByVal lDirectionToShift As dcShiftDirection) As Long
'Author: Donald Moore (MindRape)
'E-mail: moore@futureone.com
'  Date: 06/16/99
'Enters:
'   lValue as Long
'   lNumberOfBitsToShift as Long
'   lDirectionToShift as Long
'Returns:
'   Long - shifted value
'Purpose:
'   Shift the given value by the given number of bits to shift in the given direction.
'   Shifting bits to the left acts as a multiplier and to the right divides.
Public Function Shift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long, ByVal lDirectionToShift As dcShiftDirection) As Long
Const ksCallname As String = "Shift"
On Error GoTo Procedure_Error
Dim LShift As Long

    If lDirectionToShift Then 'shift left
        LShift = lValue * (2 ^ lNumberOfBitsToShift)
    Else                      'shift right
        LShift = lValue \ (2 ^ lNumberOfBitsToShift)
    End If
    
Procedure_Exit:
    Shift = LShift
    Exit Function
    
Procedure_Error:
    Err.Raise Err.Number, ksCallname, Err.Description, Err.HelpFile, Err.HelpContext
    Resume Procedure_Exit
End Function



'========================================================================================
'Public Function LShift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long) As Long
'Author: Donald Moore (MindRape)
'E-mail: moore@futureone.com
'  Date: 06/16/99
'Enters:
'   lValue as Long
'   lNumberOfBitsToShift as Long
'Returns:
'   Long - shifted value
'Purpose:
'   Shift the given value by the given number of bits left
Public Function LShift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long) As Long
Const ksCallname As String = "LShift"
On Error GoTo Procedure_Error

    LShift = Shift(lValue, lNumberOfBitsToShift, Left)
    
Procedure_Exit:
    Exit Function
    
Procedure_Error:
    Err.Raise Err.Number, ksCallname, Err.Description, Err.HelpFile, Err.HelpContext
    Resume Procedure_Exit
End Function



'========================================================================================
'Public Function RShift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long) As Long
'Author: Donald Moore (MindRape)
'E-mail: moore@futureone.com
'  Date: 06/16/99
'Enters:
'   lValue as Long
'   lNumberOfBitsToShift as Long
'Returns:
'   Long - shifted value
'Purpose:
'   Shift the given value by the given number of bits right
Public Function RShift(ByVal lValue As Long, ByVal lNumberOfBitsToShift As Long) As Long
Const ksCallname As String = "RShift"
On Error GoTo Procedure_Error

    RShift = Shift(lValue, lNumberOfBitsToShift, Right)
    
Procedure_Exit:
    Exit Function
    
Procedure_Error:
    Err.Raise Err.Number, ksCallname, Err.Description, Err.HelpFile, Err.HelpContext
    Resume Procedure_Exit
End Function
